# import logging

# debug_level = logging.CRITICAL

# parts_logger = logging.getLogger("abml.parser")
# parts_logger.addHandler(logging.FileHandler("parser.log", encoding="utf-8", mode="w"))
# parts_logger.setLevel(debug_level)
# parts_logger.propagate = False

# parts_logger = logging.getLogger("abml.abml_parts")
# parts_logger.addHandler(logging.FileHandler("parts.log", encoding="utf-8", mode="w"))
# parts_logger.setLevel(debug_level)
# parts_logger.propagate = False

# assembly_logger = logging.getLogger("abml.abml_assembly")
# assembly_logger.addHandler(logging.FileHandler("assembly.log", encoding="utf-8", mode="w"))
# assembly_logger.setLevel(debug_level)
# assembly_logger.propagate = False

# sections_logger = logging.getLogger("abml.abml_sections")
# sections_logger.addHandler(logging.FileHandler("sections.log", encoding="utf-8", mode="w"))
# sections_logger.setLevel(debug_level)
# sections_logger.propagate = False

# materials_logger = logging.getLogger("abml.abml_materials")
# materials_logger.addHandler(logging.FileHandler("materials.log", encoding="utf-8", mode="w"))
# materials_logger.setLevel(debug_level)
# materials_logger.propagate = False

# helpers_logger = logging.getLogger("abml.abml_helpers")
# helpers_logger.addHandler(logging.FileHandler("helpers.log", encoding="utf-8", mode="w"))
# helpers_logger.setLevel(debug_level)
# helpers_logger.propagate = False

# helpers_logger = logging.getLogger("abml.abml_steps")
# helpers_logger.addHandler(logging.FileHandler("steps.log", encoding="utf-8", mode="w"))
# helpers_logger.setLevel(debug_level)
# helpers_logger.propagate = False
